"""Replicates some of the `logging.Logger` API."""

import logging
import sys

from . import utils

__all__ = ["log", "debug", "info", "warning", "error", "critical", "exception"]


def log(level, message, *args, **kwargs):
    utils.create_logger_record(level, message, *args, **kwargs)


def debug(message, *args, **kwargs):
    log(logging.DEBUG, message, *args, **kwargs)


def info(message, *args, **kwargs):
    log(logging.INFO, message, *args, **kwargs)


def warning(message, *args, **kwargs):
    log(logging.WARNING, message, *args, **kwargs)


def error(message, *args, **kwargs):
    log(logging.ERROR, message, *args, **kwargs)


def critical(message, *args, **kwargs):
    log(logging.CRITICAL, message, *args, **kwargs)


def exception(message, *args, **kwargs):
    kwargs["exc_info"] = kwargs.get("exc_info", sys.exc_info())
    log(logging.ERROR, message, *args, **kwargs)



# pylint: disable=wildcard-import

from logging import CRITICAL, DEBUG, ERROR, INFO, WARNING

from .helpers import *
from .logger import *

WARN = WARNING

d = debug
i = info
w = warn = warning
e = error
c = critical
exc = exception



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

import pytest

import log


@pytest.mark.parametrize(
    "name, levelname",
    [
        ("c", "CRITICAL"),
        ("critical", "CRITICAL"),
        ("d", "DEBUG"),
        ("debug", "DEBUG"),
        ("e", "ERROR"),
        ("error", "ERROR"),
        ("exc", "ERROR"),
        ("exception", "ERROR"),
        ("i", "INFO"),
        ("info", "INFO"),
        ("w", "WARNING"),
        ("warn", "WARNING"),
        ("warning", "WARNING"),
    ],
)
def test_level_mapping(expect, caplog, name, levelname):
    getattr(log, name)("message")
    expect(caplog.records[-1].levelname) == levelname



"""Implements the "magic" to create `logging` records for the caller."""

import dataclasses
import inspect
import logging
from pprint import pformat
from typing import Dict, Tuple

from . import state


def create_logger_record(
    level, message, *args, name: str = "", exc_info=None, **kwargs
) -> bool:
    frame = inspect.currentframe().f_back.f_back.f_back  # type: ignore
    assert frame

    name, parent_name = parse_name(name, frame.f_globals)
    if parent_name in state.silenced:
        return False

    logger = get_logger(name, parent_name)
    if not logger.isEnabledFor(level):
        return False

    record = logger.makeRecord(
        name,
        level,
        fn=parse_filename(frame.f_globals),
        lno=frame.f_lineno,
        msg=format_message(message),
        args=args,
        exc_info=exc_info,
        extra=kwargs,
        sinfo=None,
    )
    logger.handle(record)
    return True


def parse_name(custom_name: str, frame_info: Dict) -> Tuple[str, str]:
    module_name = custom_name or frame_info["__name__"]
    if module_name == "__main__":
        try:
            module_name = frame_info["__file__"].split(".")[0].replace("/", ".")
        except KeyError:
            module_name = "interactive"
    parent_module_name = module_name.split(".")[0]
    return module_name, parent_module_name


def get_logger(name: str, parent_name: str):
    logger = logging.getLogger(name)
    if not logger.level:
        parent_logger = logging.getLogger(parent_name)
        logger.level = parent_logger.level
    if not logger.level:
        logger.level = logging.root.level
    return logger


def parse_filename(frame_info: Dict) -> str:
    return frame_info.get("__file__", "interactive")


def format_message(value) -> str:
    if dataclasses.is_dataclass(value):
        value = dataclasses.asdict(value)
    if not isinstance(value, str):
        value = pformat(value)
    return value



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison,disallowed-name

import logging
from dataclasses import dataclass

from log.utils import create_logger_record, format_message, parse_name


def describe_create_logger_record():
    def it_uses_the_default_log_level_for_new_loggers(expect, monkeypatch):
        monkeypatch.setattr(logging.root, "level", logging.INFO)

        expect(create_logger_record(logging.DEBUG, "hello", name="new")) == False
        expect(create_logger_record(logging.INFO, "hello", name="new")) == True
        expect(create_logger_record(logging.WARNING, "hello", name="new")) == True

    def it_inherits_the_parent_logging_level(expect):
        logger = logging.getLogger("root")
        logger.level = logging.WARNING

        expect(create_logger_record(logging.DEBUG, "hello", name="root.new")) == False
        expect(create_logger_record(logging.INFO, "hello", name="root.new")) == False
        expect(create_logger_record(logging.WARNING, "hello", name="root.new")) == True


def describe_parse_name():
    def it_uses_the_filename_when_main(expect):
        frame_info = {"__file__": "my_package/my_module.py"}
        expect(parse_name("__main__", frame_info)[0]) == "my_package.my_module"

    def it_handles_interactive_sessions(expect):
        expect(parse_name("__main__", {})[0]) == "interactive"


def describe_format_message():
    def it_formats_structures(expect):
        data = {x: x * 10 for x in range(20)}
        expect(format_message(data).count("\n")) == 19

    def it_formats_dataclasses(expect):
        @dataclass
        class Example:
            foo: int = 1
            bar: str = "abc"

        example = Example()

        expect(format_message(example)) == "{'bar': 'abc', 'foo': 1}"

    def it_preserves_strings(expect):
        expect(format_message("foobar")) == "foobar"
